----------------------------------------------------------------------------------
-- Company: 
-- Engineer: 
-- 
-- Create Date: 05/31/2014 06:33:53 PM
-- Design Name: 
-- Module Name: sys_sync_io - rtl
-- Project Name: 
-- Target Devices: 
-- Tool Versions: 
-- Description: 
-- 
-- Dependencies: 
-- 
-- Revision:
-- Revision 0.01 - File Created
-- Additional Comments:
-- 
----------------------------------------------------------------------------------


library IEEE;
use IEEE.STD_LOGIC_1164.ALL;

-- Uncomment the following library declaration if using
-- arithmetic functions with Signed or Unsigned values
--use IEEE.NUMERIC_STD.ALL;

-- Uncomment the following library declaration if instantiating
-- any Xilinx leaf cells in this code.
library UNISIM;
use UNISIM.VComponents.all;

entity sys_sync_io is
    Port ( clkin_sys : in STD_LOGIC;
           din_sys_re : in STD_LOGIC;
           din_sys_fe : in STD_LOGIC;
           din_sys_ddr : in STD_LOGIC;
           dout_sys_re : out STD_LOGIC;
           dout_sys_fe : out STD_LOGIC;
           dout_sys_ddr : out STD_LOGIC);
end sys_sync_io;

architecture rtl of sys_sync_io is

component sys_mmcm
port
 (-- Clock in ports
  clk_in1           : in     std_logic;
  -- Clock out ports
  clk_out1          : out    std_logic
 );
end component;

ATTRIBUTE SYN_BLACK_BOX : BOOLEAN;
ATTRIBUTE SYN_BLACK_BOX OF sys_mmcm : COMPONENT IS TRUE;


ATTRIBUTE BLACK_BOX_PAD_PIN : STRING;
ATTRIBUTE BLACK_BOX_PAD_PIN OF sys_mmcm : COMPONENT IS "clk_in1,clk_out1";

signal clkin_sys_ibufg : std_logic;
signal clkin_sys_bufg : std_logic;

signal reg1_sys_re : std_logic;
signal reg2_sys_re : std_logic;

signal reg1_sys_fe : std_logic;
signal reg2_sys_fe : std_logic;

signal regp1_sys_ddr : std_logic;
signal regn1_sys_ddr : std_logic;
signal regp2_sys_ddr : std_logic;
signal regn2_sys_ddr : std_logic;
signal regp3_sys_ddr : std_logic;
signal regn3_sys_ddr : std_logic;

constant USE_MMCM : boolean := TRUE;

begin

direct_clk: if not USE_MMCM generate

--ibufg_inst: IBUFG port map (
--  I => clkin_sys,
--  O => clkin_sys_ibufg );
  
--bufg_inst: BUFG port map (
--    I => clkin_sys_ibufg,
--    O => clkin_sys_bufg );
end generate;

mmcm_clk: if USE_MMCM generate
sys_mmcm_inst: sys_mmcm
   port map ( 
   -- Clock in ports
   clk_in1 => clkin_sys,
  -- Clock out ports  
   clk_out1 => clkin_sys_bufg              
 );
end generate;    
    
 

sys_sdr_re: process (clkin_sys_bufg)
begin
  if rising_edge(clkin_sys_bufg) then
    reg1_sys_re <= din_sys_re;   -- System synchronous, SDR, Rising Edge Input
    reg2_sys_re <= reg1_sys_re;
    dout_sys_re  <= reg2_sys_re; -- System synchronous, SDR, Rising Edge Output
  end if;
end process;

proc_sys_sdr_fe: process (clkin_sys_bufg)
begin
  if falling_edge(clkin_sys_bufg) then
    reg1_sys_fe <= din_sys_fe;  -- System synchronous, SDR, Falling Edge Input
    reg2_sys_fe <= reg1_sys_fe;
    dout_sys_fe  <= reg2_sys_fe;  -- System synchronous, SDR, Falling Edge Output
  end if;
end process;

IDDR_inst: IDDRE1 
   generic map (
      DDR_CLK_EDGE => "SAME_EDGE_PIPELINED" -- "OPPOSITE_EDGE", "SAME_EDGE" 
                                       -- or "SAME_EDGE_PIPELINED" 
     ) 
   port map (
      Q1 => regp1_sys_ddr, -- 1-bit output for positive edge of clock 
      Q2 => regn1_sys_ddr, -- 1-bit output for negative edge of clock
      C => clkin_sys_bufg,   -- 1-bit clock input
      CB => not clkin_sys_bufg , -- 1-bit input: Inversion of High-speed clock C
      D => din_sys_ddr,   -- 1-bit DDR data input
      R => '0'   -- 1-bit reset
      );

proc_sys_ddr: process (clkin_sys_bufg)
begin
  if rising_edge(clkin_sys_bufg) then
    regp2_sys_ddr <= regp1_sys_ddr;  
    regn2_sys_ddr <= regn1_sys_ddr;
    regp3_sys_ddr <= regp2_sys_ddr;  
    regn3_sys_ddr <= regn2_sys_ddr;
   end if;
end process;

ODDR_inst: ODDRE1
   generic map(
       SRVAL => '0') --  -- Initializes the ODDRE1 Flip-Flops to the specified value ('0', '1')
   port map (
      Q => dout_sys_ddr,   -- 1-bit DDR output
      C => clkin_sys_bufg,    -- 1-bit clock input
      D1 => regn3_sys_ddr,  -- 1-bit data input Parallel data input 1
      D2 => regp3_sys_ddr,  -- 1-bit data input Parallel data input 2
      SR => '0'    -- 1-bit input: Active High Async Reset
   );

end rtl;


--<copyright-disclaimer-start>
--<copyright-disclaimer-start>
--  **************************************************************************************************************
--  * © 2025 Advanced Micro Devices, Inc. All rights reserved.                                                   *
--  * DISCLAIMER                                                                                                 *
--  * The information contained herein is for informational purposes only, and is subject to change              *
--  * without notice. While every precaution has been taken in the preparation of this document, it              *
--  * may contain technical inaccuracies, omissions and typographical errors, and AMD is under no                *
--  * obligation to update or otherwise correct this information.  Advanced Micro Devices, Inc. makes            *
--  * no representations or warranties with respect to the accuracy or completeness of the contents of           *
--  * this document, and assumes no liability of any kind, including the implied warranties of noninfringement,  *
--  * merchantability or fitness for particular purposes, with respect to the operation or use of AMD            *
--  * hardware, software or other products described herein.  No license, including implied or                   *
--  * arising by estoppel, to any intellectual property rights is granted by this document.  Terms and           *
--  * limitations applicable to the purchase or use of AMD’s products are as set forth in a signed agreement     *
--  * between the parties or in AMD's Standard Terms and Conditions of Sale. GD-18                               *
--  *                                                                                                            *
--  **************************************************************************************************************
--<copyright-disclaimer-end>
--<copyright-disclaimer-end>
--<copyright-disclaimer-end>
