//////////////////////////////////////////////////////////////////////
////                                                              ////
////  tb_cksum_top.v                                              ////
////                                                              ////
////                                                              ////
//////////////////////////////////////////////////////////////////////

`timescale 1 ns / 1 ns

`define SOP 1
`define NOSOP 0

// Define Module for Test Fixture
module tb_cksum();

    reg         sop_data=0;
    reg [15:0]  pkt_data=0;
    reg [7:0]   valid_data=0;
    integer     i=0;
    reg         rst_i=0;
    reg         SysClk_in=0;
    reg         clk_i=0;

// system clock
   always begin
       #5 SysClk_in = ~SysClk_in;
   end

task Reset;
begin
    rst_i = 0; # 10;
    rst_i = 1; # 10;
    rst_i = 0; # 10;
end
endtask


task write_data;
    input            sop_i;
    input reg [15:0] dat_i;
    //reg [7:0] wait_cnt;
    begin

    //wait_cnt = 0;
    sop_data = sop_i;
    pkt_data = dat_i;

    // SOP doesnt contain valid L3 data, clears L3cksum
    @(posedge CKSUM_UUT.SysClk);
    if(sop_i) begin
        #1 valid_data = 8'h00;
        $display($time, " SOP found, clear cksum"); 
    end
    else begin
        #1 valid_data = 8'hFF;
        $display($time, " Writing data: %x ",  dat_i); 
    end

    // off
    @(posedge CKSUM_UUT.SysClk);
    #1 valid_data = 8'h00;
    end
endtask





// Instantiate the UUT
 cksum CKSUM_UUT ( 
    // System Signals
    .SysClk_in      (SysClk_in),           

    // Input Control
    .SOP            (sop_data),            
    .L3LaneEn       (valid_data), 
    // Input Data
    .L3PktData0     (pkt_data),     
    .L3PktData1     (pkt_data),     
    .L3PktData2     (pkt_data),     
    .L3PktData3     (pkt_data),     
    .L3PktData4     (pkt_data),     
    .L3PktData5     (pkt_data),     
    .L3PktData6     (pkt_data),     
    .L3PktData7     (pkt_data),     
    // output Data
    .cksum_valid    (),
    .L3ChksumFinal  ()
);

// Initialize Inputs
    initial begin
//      $monitor("Time: %d clk_i=%b",
//          $time, clk_i);
            clk_i = 0;
            rst_i = 0;
            pkt_data = 0;
            sop_data = 0;
            valid_data = 8'h0;
    Reset();

    $display(" \n WAITING... for MMCM Lock....");
    wait(CKSUM_UUT.mmcm_locked)
    $display(" \n MMCM Locked!");

    $display(" \n Sending L3 Packet data  \n");

    for(i=0; i < 4; i=i+1)
    begin
        if(i==0)
            write_data( `SOP, pkt_data );
        else
            write_data( `NOSOP, pkt_data );

        pkt_data = pkt_data +1;
    end

    #100;
    $display(" \n End2 of Cksum testing \n"); 
    
    $display($time, " Simulation Done \n"); $stop(1);

end

endmodule 

//<copyright-disclaimer-start>
//<copyright-disclaimer-start>
//  **************************************************************************************************************
//  * © 2025 Advanced Micro Devices, Inc. All rights reserved.                                                   *
//  * DISCLAIMER                                                                                                 *
//  * The information contained herein is for informational purposes only, and is subject to change              *
//  * without notice. While every precaution has been taken in the preparation of this document, it              *
//  * may contain technical inaccuracies, omissions and typographical errors, and AMD is under no                *
//  * obligation to update or otherwise correct this information.  Advanced Micro Devices, Inc. makes            *
//  * no representations or warranties with respect to the accuracy or completeness of the contents of           *
//  * this document, and assumes no liability of any kind, including the implied warranties of noninfringement,  *
//  * merchantability or fitness for particular purposes, with respect to the operation or use of AMD            *
//  * hardware, software or other products described herein.  No license, including implied or                   *
//  * arising by estoppel, to any intellectual property rights is granted by this document.  Terms and           *
//  * limitations applicable to the purchase or use of AMD’s products are as set forth in a signed agreement     *
//  * between the parties or in AMD's Standard Terms and Conditions of Sale. GD-18                               *
//  *                                                                                                            *
//  **************************************************************************************************************
//<copyright-disclaimer-end>
//<copyright-disclaimer-end>
//<copyright-disclaimer-end>
//<copyright-disclaimer-end>
