// ******************************************************************
// ******               Counter inferencing Example            ******
// ******************************************************************
//
// File:        counter.v 
// Revision:    1.0
// Project:     Inferencing Lab
// Company:     Xilinx
// Date:        March 12, 2013 
//
//
//---------------------------------------------------------------------  
// Description:
//---------------------------------------------------------------------  
//  This illustrates the ability of synthesis to use DSPs to
//  inference wide counters 
//---------------------------------------------------------------------  
`timescale 1ns / 1ps

`define MAX_COUNT       48'hFFFF_FFFF_FFF0
`define COMPARE_VALUE   48'hFFFF_0000_0000

(* use_dsp = "yes" *) module counter #
 (
 parameter WIDTH = 48
 )(
  input                     CLK,
  input      [WIDTH-1:0]    DATA1,
  input      [WIDTH-1:0]    DATA2,
  input                     LD_CNT,
  input                     INC_CNT,
  output reg                CMP_VALID = 1'b0,
  output reg [WIDTH-1:0]    DATA_OUT = {WIDTH{1'b0}}
);

    // regs and wires
  reg           increment    = 1'b0;
  reg           load         = 1'b0;
  reg           pattDetect   = 1'b0;
  reg [WIDTH-1:0]  datain1   = {WIDTH{1'b0}};
  reg [WIDTH-1:0]  datain2   = {WIDTH{1'b0}};
 reg [WIDTH-1:0]   count     = {WIDTH{1'b0}};


  // input registers
  always @(posedge CLK)
  begin
      datain1   <= DATA1;
      datain2   <= DATA2;
      load      <= LD_CNT;
      increment <= INC_CNT;
  end
      
  // counter 
  always @(posedge CLK)
    begin
        if (count==`MAX_COUNT)
            count <= count;
        else if (load)
           count <= datain1;
        else if (increment)
            count <= count+1;

        DATA_OUT <= count;
    end 

    // comparator
  always @(posedge CLK)
    begin
        pattDetect <= (datain2==`COMPARE_VALUE);
        CMP_VALID  <= pattDetect;
    end

endmodule


//<copyright-disclaimer-start>
//<copyright-disclaimer-start>
//  **************************************************************************************************************
//  * © 2025 Advanced Micro Devices, Inc. All rights reserved.                                                   *
//  * DISCLAIMER                                                                                                 *
//  * The information contained herein is for informational purposes only, and is subject to change              *
//  * without notice. While every precaution has been taken in the preparation of this document, it              *
//  * may contain technical inaccuracies, omissions and typographical errors, and AMD is under no                *
//  * obligation to update or otherwise correct this information.  Advanced Micro Devices, Inc. makes            *
//  * no representations or warranties with respect to the accuracy or completeness of the contents of           *
//  * this document, and assumes no liability of any kind, including the implied warranties of noninfringement,  *
//  * merchantability or fitness for particular purposes, with respect to the operation or use of AMD            *
//  * hardware, software or other products described herein.  No license, including implied or                   *
//  * arising by estoppel, to any intellectual property rights is granted by this document.  Terms and           *
//  * limitations applicable to the purchase or use of AMD’s products are as set forth in a signed agreement     *
//  * between the parties or in AMD's Standard Terms and Conditions of Sale. GD-18                               *
//  *                                                                                                            *
//  **************************************************************************************************************
//<copyright-disclaimer-end>
//<copyright-disclaimer-end>
//<copyright-disclaimer-end>
//<copyright-disclaimer-end>
