set hostOS [lindex $tcl_platform(os) 0]
if { $hostOS == "Windows" } {
    set TRAINING_PATH "c:/training"
	
} else {
    set TRAINING_PATH "/home/amd/training"

}
puts "$TRAINING_PATH"

set projDir $TRAINING_PATH/revcntrl/lab/KCU105

# Create project
create_project -force uart_led $projDir/uart_led_subsystem -part xcku040-ffva1156-2-e

# Set project properties
set obj [get_projects uart_led]
set_property "board_part" "xilinx.com:kcu105:part0:1.7" $obj
set_property "simulator_language" "Mixed" $obj
set_property "target_language" "Verilog" $obj
#Setting the custom IP repository path
set_property  ip_repo_paths  $TRAINING_PATH/revcntrl/lab/KCU105/cip [current_project]
update_ip_catalog

# Create block design
create_bd_design "uart_led"
#Adding uart_baud_gen, uart_rx_ctl_led_ctl and meta_harden IPs
create_bd_cell -type ip -vlnv xilinxtraining.com:ip:uart_baud_gen:1.0 uart_baud_gen_0
create_bd_cell -type ip -vlnv xilinxtraining.com:ip:uart_rx_ctl:1.0 uart_rx_ctl_0
create_bd_cell -type ip -vlnv xilinxtraining.com:ip:led_ctl:1.0 led_ctl_0
create_bd_cell -type ip -vlnv xilinxtraining.com:ip:meta_harden:1.0 meta_harden_0
create_bd_cell -type ip -vlnv xilinxtraining.com:ip:meta_harden:1.0 meta_harden_1
create_bd_cell -type ip -vlnv xilinxtraining.com:ip:meta_harden:1.0 meta_harden_2
#Adding utility buffer IP and customizing
create_bd_cell -type ip -vlnv xilinx.com:ip:util_ds_buf:2.2 util_ds_buf_0
startgroup
set_property -dict [list CONFIG.C_BUF_TYPE {IBUFDS}] [get_bd_cells util_ds_buf_0]
endgroup
#Making the IP pins external and naming them


make_bd_pins_external  [get_bd_pins util_ds_buf_0/IBUF_DS_P] [get_bd_pins util_ds_buf_0/IBUF_DS_N]
set_property name clk_pin_p [get_bd_ports IBUF_DS_P_0]
set_property name clk_pin_n [get_bd_ports IBUF_DS_N_0]
regenerate_bd_layout

make_bd_pins_external  [get_bd_pins led_ctl_0/led_o]
set_property name led_pins [get_bd_ports led_o_0]

set_property name meta_harden_btn [get_bd_cells meta_harden_1]
make_bd_pins_external  [get_bd_pins meta_harden_btn/signal_src]
set_property name btn_pin [get_bd_ports signal_src_0]

set_property name meta_harden_rst [get_bd_cells meta_harden_2]
make_bd_pins_external  [get_bd_pins meta_harden_rst/signal_src]
set_property name rst_pin [get_bd_ports signal_src_0]

make_bd_pins_external  [get_bd_pins meta_harden_0/signal_src]
set_property name rxd_pin [get_bd_ports signal_src_0]
#Clock connection to all the IPs
connect_bd_net [get_bd_pins util_ds_buf_0/IBUF_OUT] [get_bd_pins meta_harden_0/clk_dst]
connect_bd_net [get_bd_pins util_ds_buf_0/IBUF_OUT] [get_bd_pins led_ctl_0/clk_rx]
connect_bd_net [get_bd_pins util_ds_buf_0/IBUF_OUT] [get_bd_pins uart_rx_ctl_0/clk_rx]
connect_bd_net [get_bd_pins util_ds_buf_0/IBUF_OUT] [get_bd_pins meta_harden_rst/clk_dst]
connect_bd_net [get_bd_pins util_ds_buf_0/IBUF_OUT] [get_bd_pins meta_harden_btn/clk_dst]
connect_bd_net [get_bd_pins util_ds_buf_0/IBUF_OUT] [get_bd_pins uart_baud_gen_0/clk]
#Reset connection to all the IPs
connect_bd_net [get_bd_pins meta_harden_btn/rst_dst] [get_bd_pins meta_harden_rst/signal_dst]
connect_bd_net [get_bd_pins meta_harden_rst/signal_dst] [get_bd_pins meta_harden_0/rst_dst]
connect_bd_net [get_bd_pins meta_harden_rst/signal_dst] [get_bd_pins uart_rx_ctl_0/rst_clk_rx]
connect_bd_net [get_bd_pins meta_harden_rst/signal_dst] [get_bd_pins uart_baud_gen_0/rst]
connect_bd_net [get_bd_pins meta_harden_rst/signal_dst] [get_bd_pins led_ctl_0/rst_clk_rx]
#Interconnection between the IPs
connect_bd_net [get_bd_pins meta_harden_0/signal_dst] [get_bd_pins uart_rx_ctl_0/rxd_clk_rx]
connect_bd_net [get_bd_pins meta_harden_btn/signal_dst] [get_bd_pins led_ctl_0/btn_clk_rx]
connect_bd_net [get_bd_pins uart_rx_ctl_0/rx_data] [get_bd_pins led_ctl_0/rx_data]
connect_bd_net [get_bd_pins uart_rx_ctl_0/rx_data_rdy] [get_bd_pins led_ctl_0/rx_data_rdy]
connect_bd_net [get_bd_pins uart_rx_ctl_0/baud_x16_en] [get_bd_pins uart_baud_gen_0/baud_x16_en]
group_bd_cells uart_rx_i0 [get_bd_cells uart_baud_gen_0] [get_bd_cells meta_harden_0] [get_bd_cells uart_rx_ctl_0]

#Addding constant IP and customizing
create_bd_cell -type ip -vlnv xilinx.com:ip:xlconstant:1.1 xlconstant_0
set_property -dict [list CONFIG.CONST_VAL {0}] [get_bd_cells xlconstant_0]
connect_bd_net [get_bd_pins meta_harden_rst/rst_dst] [get_bd_pins xlconstant_0/dout]
regenerate_bd_layout
#Validates the block design for errors
validate_bd_design

#<copyright-disclaimer-start>
#<copyright-disclaimer-start>
#  **************************************************************************************************************
#  * © 2025 Advanced Micro Devices, Inc. All rights reserved.                                                   *
#  * DISCLAIMER                                                                                                 *
#  * The information contained herein is for informational purposes only, and is subject to change              *
#  * without notice. While every precaution has been taken in the preparation of this document, it              *
#  * may contain technical inaccuracies, omissions and typographical errors, and AMD is under no                *
#  * obligation to update or otherwise correct this information.  Advanced Micro Devices, Inc. makes            *
#  * no representations or warranties with respect to the accuracy or completeness of the contents of           *
#  * this document, and assumes no liability of any kind, including the implied warranties of noninfringement,  *
#  * merchantability or fitness for particular purposes, with respect to the operation or use of AMD            *
#  * hardware, software or other products described herein.  No license, including implied or                   *
#  * arising by estoppel, to any intellectual property rights is granted by this document.  Terms and           *
#  * limitations applicable to the purchase or use of AMD’s products are as set forth in a signed agreement     *
#  * between the parties or in AMD's Standard Terms and Conditions of Sale. GD-18                               *
#  *                                                                                                            *
#  **************************************************************************************************************
#<copyright-disclaimer-end>
#<copyright-disclaimer-end>
#<copyright-disclaimer-end>
#<copyright-disclaimer-end>


#Generate output product for block design
generate_target all [get_files  $TRAINING_PATH/revcntrl/lab/KCU105/uart_led_subsystem/uart_led.srcs/sources_1/bd/uart_led/uart_led.bd]
save_bd_design
