###########################################################################
## Baselining Completer Script
###########################################################################

# load the standard helper file
source ../../fpgaSupport_scripts/script1.tcl
source ../../fpgaSupport_scripts/script2.tcl

# project constants
set verbose 	1
set tcName 	Baselining
set demoOrLab 	completed
set projName 	uart_led

## *********** Step 1 : Copying the project ***********
proc copyProject {} {

variable platform
variable language
variable tcName
variable demoOrLab 
variable projName 
variable TRAINING_PATH

# Set variables for source and destination paths
set sourcePath $TRAINING_PATH/$tcName/lab/$platform
set destPath $TRAINING_PATH/$tcName/completed/$platform

# Create the destination directories if they don't exist
file mkdir $destPath

# Copy the zip file to the destination directory
file copy -force $sourcePath/$language.zip $destPath
puts "sourcePath: $sourcePath"
puts "destPath: $destPath"

exec unzip $TRAINING_PATH/$tcName/completed/$platform/$language.zip -d $TRAINING_PATH/$tcName/completed/$platform/$language
}
 ## *********** Step 2 : Opening a project ***********

proc openProject {} {
variable platform
variable language
variable tcName
variable demoOrLab 
variable projName 
variable TRAINING_PATH

# Add the platform and language combination that you want 
set isLangNotSelected [string compare -nocase $language "undefined"]
set isPlatNotSelected [string compare -nocase $platform "undefined"]
   
if {$isLangNotSelected} {
      puts "Please type: use VHDL | Verilog"
      puts "   then rerun the projectCreate"
} 
if {$isPlatNotSelected} {
      puts "Please type: use KCU105 | KC705 | KC7xx"
      puts "   then rerun the projectCreate"
}

# Open a project
set projName.xpr {append $projName .xpr}
open_project $TRAINING_PATH/$tcName/$demoOrLab/$platform/$language/$projName.xpr
}


 ## *********** Step 3 : Open synthesized design and run report_clock_networks and add primary clock constraints and run report_clock_networks again ***********

proc addPriConstr {} {
variable platform
variable language
variable tcName
variable demoOrLab 
variable projName
variable TRAINING_PATH

# Open the synthesized design
open_run synth_1 -name synth_1

report_clock_networks -name {network_1}

set projName.srcs {append $projName .srcs}
set_property target_constrs_file $TRAINING_PATH/$tcName/$demoOrLab/$platform/$language/$projName.srcs/constrs_1/imports/uart_led_Baselining.xdc [current_fileset -constrset]

# Adding the Primary Clock Constraints for both 7 series and Ultra-Scale board
create_clock -period 8 -name clk_pin_p -waveform {0.000 1.667} [get_ports clk_pin_p]

save_constraints

# Running the report_clock_networks
refresh_design
report_clock_networks -name {network_1}
report_clock_interaction -delay_type min_max -significant_digits 3 -name timing_1
check_timing -help

# Running check_timing 
check_timing -verbose

# Running report_timing_summary
report_timing_summary -delay_type min_max -report_unconstrained -check_timing_verbose -max_paths 10 -input_pins -name timing_1
close_design
}

 
 ## *********** Step 4 : Implementing the design and run report_timing_summary ***********

proc genReport {} {
variable platform
variable language
variable tcName
variable demoOrLab 
variable projName

# Implementing the design
implementationRun

# Running report_timing_summary and view different sections of the report
report_timing_summary -delay_type min_max -report_unconstrained -check_timing_verbose -max_paths 10 -input_pins -name timing_1
}


 ## *********** Step 5 : Adding the IO constraints and Implementing the design and run report_timing_summary again ***********

proc addIOConstr {} {
variable platform
variable language
variable tcName
variable demoOrLab 
variable projName

# Adding the Constraints for both 7 series and Ultra-Scale board

set_input_delay -clock [get_clocks clk_pin_p] 1.500 [get_ports {rst_pin rxd_pin btn_pin}]
set_input_delay -clock [get_clocks clk_pin_p] -min 1.00 [get_ports {rst_pin rxd_pin btn_pin}]
set_output_delay -clock [get_clocks clk_pin_p] 0.500 [get_ports led_pins*]
set_output_delay -clock [get_clocks clk_pin_p] -min -0.500 [get_ports led_pins*]

save_constraints -force

reset_run impl_1

# Implementing the design again
implementationRun

# Running check_timing
check_timing

}

 ## ********** Running only the steps that are required with Make **************

proc make {stopAt} {

   puts "Running until the step $stopAt"
   set limit [string tolower $stopAt]
   switch $limit {
      step1  { copyProject }
	  step2  { make step1;openProject }
      step3  { make step2; addPriConstr }
      step4  { make step3; genReport }
      step5  { make step4; addIOConstr }
	  all    { make step5 }
      default { 
         puts "Call the make proc, Should be make step*" 
			  }	
	}	
}


#<copyright-disclaimer-start>
#<copyright-disclaimer-start>
#  **************************************************************************************************************
#  * © 2025 Advanced Micro Devices, Inc. All rights reserved.                                                   *
#  * DISCLAIMER                                                                                                 *
#  * The information contained herein is for informational purposes only, and is subject to change              *
#  * without notice. While every precaution has been taken in the preparation of this document, it              *
#  * may contain technical inaccuracies, omissions and typographical errors, and AMD is under no                *
#  * obligation to update or otherwise correct this information.  Advanced Micro Devices, Inc. makes            *
#  * no representations or warranties with respect to the accuracy or completeness of the contents of           *
#  * this document, and assumes no liability of any kind, including the implied warranties of noninfringement,  *
#  * merchantability or fitness for particular purposes, with respect to the operation or use of AMD            *
#  * hardware, software or other products described herein.  No license, including implied or                   *
#  * arising by estoppel, to any intellectual property rights is granted by this document.  Terms and           *
#  * limitations applicable to the purchase or use of AMD’s products are as set forth in a signed agreement     *
#  * between the parties or in AMD's Standard Terms and Conditions of Sale. GD-18                               *
#  *                                                                                                            *
#  **************************************************************************************************************
#<copyright-disclaimer-end>
#<copyright-disclaimer-end>
#<copyright-disclaimer-end>
#<copyright-disclaimer-end>
