###########################################################################
## Debug Flow in an IP Integrator Block Design Completer Script
###########################################################################

# load the standard helper file
source -quiet ../../fpgaSupport_scripts/script1.tcl
source -quiet ../../fpgaSupport_scripts/script2.tcl
# project constants
set verbose 	1
set tcName 	Dbg_IPI
set demoOrLab 	completed
set projName 	uart_rx
 ## *********** Step 1 : Copying the project ***********
proc copyProject {} {

variable platform
variable language
variable tcName
variable demoOrLab 
variable projName 
variable TRAINING_PATH

# Set variables for source and destination paths
set sourcePath $TRAINING_PATH/$tcName/lab/$platform
set destPath $TRAINING_PATH/$tcName/completed/$platform

# Create the destination directories if they don't exist
file mkdir $destPath

# Copy the zip file to the destination directory
file copy -force $sourcePath/$language.zip $destPath
puts "sourcePath: $sourcePath"
puts "destPath: $destPath"

exec unzip $TRAINING_PATH/$tcName/completed/$platform/$language.zip -d $TRAINING_PATH/$tcName/completed/$platform/$language
}
 ## *********** Step 1 : Adding the Debug IP Block to the Design ***********

proc openProject {} {

variable tcName
variable demoOrLab 
variable projName 
variable platform
variable language
variable TRAINING_PATH

# Open a Example project
set projName.xpr {append $projName .xpr}
open_project $TRAINING_PATH/$tcName/$demoOrLab/$platform/$language/$projName.xpr

# open block design
open_bd_design $TRAINING_PATH/$tcName/$demoOrLab/$platform/$language/uart_rx.srcs/sources_1/bd/uart_rx_i0/uart_rx_i0.bd

 
# regenerate layout
regenerate_bd_layout

# Add the ILA core debug IP to the existing block design
create_bd_cell -type ip -vlnv xilinx.com:ip:ila:6.2 ila_0
 
}

 ## ********** Step 2 : Connecting the ILA Probe to Debug Nets **************

proc ConnectingILAProbetoDebugNets {} {

variable platform

set_property -dict [list \
  CONFIG.C_DATA_DEPTH {32768} \
  CONFIG.C_MONITOR_TYPE {Native} \
  CONFIG.C_NUM_OF_PROBES {4} \
  CONFIG.C_PROBE0_WIDTH {8} \
] [get_bd_cells ila_0]

set_property location {3 745 632} [get_bd_cells ila_0]
connect_bd_net [get_bd_pins util_ds_buf_0/IBUF_OUT] [get_bd_pins ila_0/clk]
connect_bd_net [get_bd_pins ila_0/probe0] [get_bd_pins uart_rx_i0/uart_rx_ctl_0/rx_data]
connect_bd_net [get_bd_pins ila_0/probe1] [get_bd_pins uart_rx_i0/uart_rx_ctl_0/rx_data_rdy]
connect_bd_net [get_bd_pins ila_0/probe2] [get_bd_pins uart_rx_i0/meta_harden_0/signal_dst]
connect_bd_net [get_bd_ports rxd_pin] [get_bd_pins ila_0/probe3]
regenerate_bd_layout

# Run a manual design validation
validate_bd_design
 
# save Block Design
save_bd_design
#  Close block design 
close_bd_design [get_bd_designs uart_rx_i0]
}

 ## ********** Step 3 : Generating the Bitstream and Analyzing the Design **************
 
proc GenerateBitstream {} {
    variable tcName
    variable demoOrLab
    variable projName
    variable language
    variable platform
    variable TRAINING_PATH

    # Create a wrapper for the block design
    make_wrapper -files [get_files /home/amd/training/Dbg_IPI/completed/ZCU104/vhdl/uart_rx.srcs/sources_1/bd/uart_rx_i0/uart_rx_i0.bd] -top

    add_files -norecurse /home/amd/training/Dbg_IPI/completed/ZCU104/vhdl/uart_rx.gen/sources_1/bd/uart_rx_i0/hdl/uart_rx_i0_wrapper.vhd
    update_compile_order -fileset sources_1
    update_compile_order -fileset sources_1

    reset_run uart_rx_i0_meta_harden_0_0_synth_1
    reset_run uart_rx_i0_uart_baud_gen_0_0_synth_1
    reset_run uart_rx_i0_uart_rx_ctl_0_0_synth_1
    reset_run uart_rx_i0_meta_harden_1_0_synth_1
    reset_run uart_rx_i0_meta_harden_2_0_synth_1

    # Set the HDL wrapper as the top module
    set_property top uart_rx_i0_wrapper [current_fileset]

        # Generate the bitstream
    launch_runs impl_1 -to_step write_bitstream -jobs 4
}



 
# Manual Step : Open the hardware manager and program the board
# Manual Step : Launch the tera term and apply input as per instructions  
 
 ## ********** Running through the steps that are required with Make **************

proc make {stopAt} {

   puts "Running until the step $stopAt"
   set limit [string tolower $stopAt]
   switch $limit {
      step1  { copyProject }
	  step2  { make step1; openProject }
      step3  { make step2; ConnectingILAProbetoDebugNets }
	  step4  { make step3; GenerateBitstream }
      all    { make step4 }
      default { 
         puts "Call the make proc, Should be make step*" 
			  }	
	}	
}

#<copyright-disclaimer-start>
#<copyright-disclaimer-start>
#  **************************************************************************************************************
#  * © 2025 Advanced Micro Devices, Inc. All rights reserved.                                                   *
#  * DISCLAIMER                                                                                                 *
#  * The information contained herein is for informational purposes only, and is subject to change              *
#  * without notice. While every precaution has been taken in the preparation of this document, it              *
#  * may contain technical inaccuracies, omissions and typographical errors, and AMD is under no                *
#  * obligation to update or otherwise correct this information.  Advanced Micro Devices, Inc. makes            *
#  * no representations or warranties with respect to the accuracy or completeness of the contents of           *
#  * this document, and assumes no liability of any kind, including the implied warranties of noninfringement,  *
#  * merchantability or fitness for particular purposes, with respect to the operation or use of AMD            *
#  * hardware, software or other products described herein.  No license, including implied or                   *
#  * arising by estoppel, to any intellectual property rights is granted by this document.  Terms and           *
#  * limitations applicable to the purchase or use of AMD’s products are as set forth in a signed agreement     *
#  * between the parties or in AMD's Standard Terms and Conditions of Sale. GD-18                               *
#  *                                                                                                            *
#  **************************************************************************************************************
#<copyright-disclaimer-end>
#<copyright-disclaimer-end>
#<copyright-disclaimer-end>
#<copyright-disclaimer-end>
