###########################################################################
## Inference Completer Script
########################################################################

# load the standard helper file
source ../../fpgaSupport_scripts/script1.tcl
source ../../fpgaSupport_scripts/script2.tcl

# project constants
set verbose 	1
set tcName 	Inference
set demoOrLab 	completed
set projNames [list "srl" "counter"]
#copyProject
 ## *********** Step 1 : Copying the project ***********
proc copyProject {} {
    variable platform
    variable language
    variable tcName
    variable demoOrLab
    variable projNames
    variable TRAINING_PATH

    # Set variables for source and destination paths
    set sourcePath $TRAINING_PATH/$tcName/lab/$platform
    set destPath $TRAINING_PATH/$tcName/completed/$platform

    # Create the destination directories if they don't exist
    file mkdir $destPath

    # Iterate over each project name
    foreach projName $projNames {
        # Copy the zip file to the destination directory
        file copy -force $sourcePath/$projName.zip $destPath
        puts "sourcePath: $sourcePath"
        puts "destPath: $destPath"

        # Unzip the copied file
        set unzipPath $destPath/$projName
        exec unzip $destPath/$projName.zip -d $unzipPath

        # Print the unzipPath
        puts "unzipPath: $unzipPath"
    }
}

##################################################################
## Step 2: Examining a Pipelined Project with Asynchronous Resets
##################################################################
proc asyncReset {} {
 variable platform
 variable tcName
 variable demoOrLab 
 variable projName 
 variable TRAINING_PATH

 set projName 	srl
 set projName.srcs {append $projName .srcs}
 set projName.xpr {append $projName .xpr}

 # Add the platform that you want 
  set isPlatNotSelected [string compare -nocase $platform "undefined"]
   
 if {$isPlatNotSelected} {
      puts "Please type: use KCU105 | KC705 | KC7xx"
      puts "   then rerun the projectCreate"
   }

 # Open a project
 open_project $TRAINING_PATH/$tcName/$demoOrLab/$platform/$projName/$projName.xpr
 
 # Open implemented design
 open_run impl_1
 
 # Generate the schematic for input and output pipelining of the DSP
 source $TRAINING_PATH/$tcName/support/pipe_schematic.tcl
 
 # Generate utilization report, timing summary report and power report
 report_utilization -name utilization_1
 report_timing_summary -delay_type min_max -report_unconstrained -check_timing_verbose -max_paths 10 -input_pins -name timing_1
 report_power -name {power_1}
}

##################################################################
## Step 3: Removing the Resets and Evaluating the Results
################################################################## 
proc noAsyncReset {} {
 variable platform
 variable tcName
 variable demoOrLab 
 variable projName 
 variable fileName     
 variable TRAINING_PATH
        
 set projName.srcs {append $projName .srcs}
 set completedfilePath $TRAINING_PATH/$tcName/support/completed_files              
 set path $TRAINING_PATH/$tcName/$demoOrLab/$platform/$projName/$projName.srcs/sources_1/imports             
  
 # Load all the completed file as the working file              
 file copy -force $completedfilePath/srl.v $path/srl.v
 
 reset_run impl_1
 
 #Running implementation process
 implementationRun
 
 #Generate the schematic for input and output pipelining of the DSP
source $TRAINING_PATH/$tcName/support/pipe_schematic_after.tcl

 #Generate utilization report, timing summary report and power report
 report_utilization -name utilization_1
 report_timing_summary -delay_type min_max -report_unconstrained -check_timing_verbose -max_paths 10 -input_pins -name timing_1
 report_power -name {power_1}
}

##################################################################
## Step 4: Inferring DSPs for Other Functions
##################################################################

proc inferDSP {} {
 variable platform
 variable tcName
 variable demoOrLab 
 variable projName 
 variable TRAINING_PATH

 set projName counter
 set projName.srcs {append $projName .srcs}
 set projName.xpr {append $projName .xpr}

 open_project $TRAINING_PATH/$tcName/$demoOrLab/$platform/$projName/$projName.xpr
 open_run synth_1 -name synth_1

 select [get_cells -filter {REF_NAME==DSP48E2}]
}
 
######################################################
## Running only the steps that are required with Make
#######################################################

proc make {stopAt} {

   puts "Running until the step $stopAt"
   set limit [string tolower $stopAt]
   switch $limit {
      step1  { copyProject }
	  step2  { make step1; asyncReset }
      step3  { make step2; noAsyncReset }
      step4  { make step3; inferDSP }
      all    { make step4 }
      default { 
         puts "Call the make proc, Should be make step*" 
			  }	
	}	
}
 

#<copyright-disclaimer-start>
#<copyright-disclaimer-start>
#  **************************************************************************************************************
#  * © 2025 Advanced Micro Devices, Inc. All rights reserved.                                                   *
#  * DISCLAIMER                                                                                                 *
#  * The information contained herein is for informational purposes only, and is subject to change              *
#  * without notice. While every precaution has been taken in the preparation of this document, it              *
#  * may contain technical inaccuracies, omissions and typographical errors, and AMD is under no                *
#  * obligation to update or otherwise correct this information.  Advanced Micro Devices, Inc. makes            *
#  * no representations or warranties with respect to the accuracy or completeness of the contents of           *
#  * this document, and assumes no liability of any kind, including the implied warranties of noninfringement,  *
#  * merchantability or fitness for particular purposes, with respect to the operation or use of AMD            *
#  * hardware, software or other products described herein.  No license, including implied or                   *
#  * arising by estoppel, to any intellectual property rights is granted by this document.  Terms and           *
#  * limitations applicable to the purchase or use of AMD’s products are as set forth in a signed agreement     *
#  * between the parties or in AMD's Standard Terms and Conditions of Sale. GD-18                               *
#  *                                                                                                            *
#  **************************************************************************************************************
#<copyright-disclaimer-end>
#<copyright-disclaimer-end>
#<copyright-disclaimer-end>
#<copyright-disclaimer-end>
