###########################################################################
## Netlist Insertion Flow Completer Script
###########################################################################

# load the standard helper file
source -quiet ../../fpgaSupport_scripts/script1.tcl
source -quiet ../../fpgaSupport_scripts/script2.tcl

# project constants
set verbose 	1
set tcName 	NetlistInsert_Flow
set demoOrLab 	completed
set projName 	uart_led

 ## *********** Step 1 : Copying the project ***********
proc copyProject {} {
   variable platform
   variable language
   variable tcName
   variable demoOrLab 
   variable projName 
   variable TRAINING_PATH

   # Set variables for source and destination paths
   set sourcePath $TRAINING_PATH/$tcName/lab/$platform
   set destPath $TRAINING_PATH/$tcName/completed/$platform

   # Create the destination directories if they don't exist
   file mkdir $destPath

   # Copy the zip file to the destination directory
   file copy -force $sourcePath/$language.zip $destPath
   puts "sourcePath: $sourcePath"
   puts "destPath: $destPath"

   exec unzip $TRAINING_PATH/$tcName/completed/$platform/$language.zip -d $TRAINING_PATH/$tcName/completed/$platform/$language
}

 ## Step 2 & 3: Opening project, generating bitstream and adding the Debug core
proc addDebugCore {} {
   variable platform
   variable language
   variable tcName
   variable demoOrLab 
   variable projName 
   variable fileName   
   variable TRAINING_PATH

   set projName.xpr "$projName.xpr"
   # Add the platform and language combination that you want 
   set isLangNotSelected [string compare -nocase $language "undefined"]
   set isPlatNotSelected [string compare -nocase $platform "undefined"] 
   if {$isLangNotSelected} {
      puts "Please type: use VHDL | Verilog"
      puts "   then rerun the projectCreate"
   } elseif {$isPlatNotSelected} {
      puts "Please type: use KCU105 | KC705 | KC7xx | ZCU104"
      puts "   then rerun the projectCreate"
   }

   # Open a project
   open_project $TRAINING_PATH/$tcName/$demoOrLab/$platform/$language/$projName.xpr
   
   # Implementing and generating bitstream
   bitstreamRun
   
   # Opening synthesized design             
   link_design -name netlist_1
   
   # Marking the nets to debug
   set_property mark_debug true [get_nets [list rx_data_rdy]]
   set_property mark_debug true [get_nets [list uart_rx_i0/meta_harden_rxd_i0_n_0]]
   set_property mark_debug true [get_nets [list {rx_data[0]} {rx_data[1]} {rx_data[2]} {rx_data[3]} {rx_data[5]} {rx_data[4]} {rx_data[6]} {rx_data[7]}]]
   
   # Creating debug core and setting its property
   create_debug_core u_ila_0 ila
   set_property C_DATA_DEPTH 32768 [get_debug_cores u_ila_0]
   set_property C_TRIGIN_EN false [get_debug_cores u_ila_0]
   set_property C_TRIGOUT_EN false [get_debug_cores u_ila_0]
   set_property C_ADV_TRIGGER false [get_debug_cores u_ila_0]
   set_property C_INPUT_PIPE_STAGES 0 [get_debug_cores u_ila_0]
   set_property C_EN_STRG_QUAL false [get_debug_cores u_ila_0]
   set_property ALL_PROBE_SAME_MU true [get_debug_cores u_ila_0]
   set_property ALL_PROBE_SAME_MU_CNT 1 [get_debug_cores u_ila_0]
   startgroup 
   set_property C_EN_STRG_QUAL true [get_debug_cores u_ila_0 ]
   set_property ALL_PROBE_SAME_MU true [get_debug_cores u_ila_0 ]
   set_property ALL_PROBE_SAME_MU_CNT 2 [get_debug_cores u_ila_0 ]
   endgroup
   set_property port_width 1 [get_debug_ports u_ila_0/clk]
   
   connect_debug_port u_ila_0/clk [get_nets [list clk_rx ]]
   
   set_property port_width 8 [get_debug_ports u_ila_0/probe0]
   set_property PROBE_TYPE DATA_AND_TRIGGER [get_debug_ports u_ila_0/probe0]
   connect_debug_port u_ila_0/probe0 [get_nets [list {rx_data[0]} {rx_data[1]} {rx_data[2]} {rx_data[3]} {rx_data[4]} {rx_data[5]} {rx_data[6]} {rx_data[7]} ]]
   create_debug_port u_ila_0 probe
   set_property port_width 1 [get_debug_ports u_ila_0/probe1]
   set_property PROBE_TYPE DATA_AND_TRIGGER [get_debug_ports u_ila_0/probe1]
   connect_debug_port u_ila_0/probe1 [get_nets [list uart_rx_i0/meta_harden_rxd_i0_n_0]]
   create_debug_port u_ila_0 probe
   set_property port_width 1 [get_debug_ports u_ila_0/probe2]
   set_property PROBE_TYPE DATA_AND_TRIGGER [get_debug_ports u_ila_0/probe2]
   connect_debug_port u_ila_0/probe2 [get_nets [list rx_data_rdy]]
   
   set_property target_constrs_file $TRAINING_PATH/$tcName/$demoOrLab/$platform/$language/uart_led.srcs/constrs_1/imports/uart_led_NetlistInsertion_Flow.xdc [current_fileset -constrset]
   save_constraints -force
   
   # Reimplementing the design and generating bitstream
   launch_runs impl_1 -to_step write_bitstream -jobs 8
}

##########################################################
# Running through the steps that are required, with make
##########################################################
proc make {stopAt} {
   puts "Running until the step $stopAt"
   set limit [string tolower $stopAt]
   switch $limit {
      step1  { copyProject }
      step2  { make step1; addDebugCore }
      all    { make step2 }
      
      default { 
         puts "Call the make proc, Should be make step*" 
      }
   }
}

# Run the make procedure with the desired step


#<copyright-disclaimer-start>
#<copyright-disclaimer-start>
#  **************************************************************************************************************
#  * © 2025 Advanced Micro Devices, Inc. All rights reserved.                                                   *
#  * DISCLAIMER                                                                                                 *
#  * The information contained herein is for informational purposes only, and is subject to change              *
#  * without notice. While every precaution has been taken in the preparation of this document, it              *
#  * may contain technical inaccuracies, omissions and typographical errors, and AMD is under no                *
#  * obligation to update or otherwise correct this information.  Advanced Micro Devices, Inc. makes            *
#  * no representations or warranties with respect to the accuracy or completeness of the contents of           *
#  * this document, and assumes no liability of any kind, including the implied warranties of noninfringement,  *
#  * merchantability or fitness for particular purposes, with respect to the operation or use of AMD            *
#  * hardware, software or other products described herein.  No license, including implied or                   *
#  * arising by estoppel, to any intellectual property rights is granted by this document.  Terms and           *
#  * limitations applicable to the purchase or use of AMD’s products are as set forth in a signed agreement     *
#  * between the parties or in AMD's Standard Terms and Conditions of Sale. GD-18                               *
#  *                                                                                                            *
#  **************************************************************************************************************
#<copyright-disclaimer-end>
#<copyright-disclaimer-end>
#<copyright-disclaimer-end>
#<copyright-disclaimer-end>
