###########################################################################
## Pipelining Completer Script
###########################################################################

# load the standard helper file
source -quiet ../../fpgaSupport_scripts/script1.tcl
source -quiet ../../fpgaSupport_scripts/script2.tcl

# project constants
set verbose 1
set tcName 		Pipelining
set demoOrLab 	completed
set projName 	cksum


 ## *********** Step 1 : Copying the project ***********
proc copyProject {} {

variable platform
variable language
variable tcName
variable demoOrLab 
variable projName 
variable TRAINING_PATH

# Set variables for source and destination paths
set sourcePath $TRAINING_PATH/$tcName/lab/$platform
set destPath $TRAINING_PATH/$tcName/completed/$platform

# Create the destination directories if they don't exist
file mkdir $destPath

# Copy the zip file to the destination directory
file copy -force $sourcePath/cksum.zip $destPath
puts "sourcePath: $sourcePath"
puts "destPath: $destPath"

exec unzip $TRAINING_PATH/$tcName/completed/$platform/$projName.zip -d $TRAINING_PATH/$tcName/completed/$platform/$projName
}

##############################################
## Step 2: Examining Non-Pipelined Design
##############################################
proc nonPipeline {} {
 variable platform
 variable tcName
 variable demoOrLab 
 variable projName 
 variable TRAINING_PATH

 set projName.xpr {append $projName .xpr}

 # Add the platform that you want 
 set isPlatNotSelected [string compare -nocase $platform "undefined"]
   
  if {$isPlatNotSelected} {
      puts "Please type: use KCU105 | KC705 | KC7xx"
      puts "   then rerun the projectCreate"
   }

 # Open a project
 open_project $TRAINING_PATH/$tcName/$demoOrLab/$platform/$projName/$projName.xpr

 # Opening the synthesized design
 open_run synth_1 -name synth_1

 # Generating Timing Summary Report
 report_timing_summary -delay_type min_max -report_unconstrained -check_timing_verbose -max_paths 10 -input_pins -name timing_1

}

##########################################################
## Step 3: Examining Partially Pipelined Design
##########################################################
proc partialPipeline {} {

 # Close the previously opened synthesized design
 close_design   

 # Make the synth_pipe1 design run the active run 
 # Design with a single additional pipeline register stage
 current_run [get_runs synth_pipe1]

 # Opening the synthesized design
 open_run synth_pipe1 -name synth_pipe1

 # Generating Timing Summary Report
 report_timing_summary -delay_type min_max -report_unconstrained -check_timing_verbose -max_paths 10 -input_pins -name timing_1
   
}


##########################################################
## Step 4: Examining Fully Pipelined Design
##########################################################

proc fullPipeline {} {

 # Close the previously opened synthesized design
 close_design 
 
 # Make the synth_pipe1 design run the active run 
 # Design with three pipeline register stages
 current_run [get_runs synth_pipe3]

 # Opening the synthesized design
 open_run synth_pipe3 -name synth_pipe3

 # Generating Timing Summary Report
 report_timing_summary -delay_type min_max -report_unconstrained -check_timing_verbose -max_paths 10 -input_pins -name timing_1
	
}

##########################################################
# Running through the steps that are required, with make
##########################################################
proc make {stopAt} {
   puts "Running until the step $stopAt"
   set limit [string tolower $stopAt]
   switch $limit {
      step1  { copyProject }
	  step2  { make step1; nonPipeline }
      step3  { make step2; partialPipeline }
      step4  { make step3; fullPipeline }
      all    { make step4 }
      
      default { 
         puts "Call the make proc, Should be make step*" 
			  }	
	}	
}

#<copyright-disclaimer-start>
#<copyright-disclaimer-start>
#  **************************************************************************************************************
#  * © 2025 Advanced Micro Devices, Inc. All rights reserved.                                                   *
#  * DISCLAIMER                                                                                                 *
#  * The information contained herein is for informational purposes only, and is subject to change              *
#  * without notice. While every precaution has been taken in the preparation of this document, it              *
#  * may contain technical inaccuracies, omissions and typographical errors, and AMD is under no                *
#  * obligation to update or otherwise correct this information.  Advanced Micro Devices, Inc. makes            *
#  * no representations or warranties with respect to the accuracy or completeness of the contents of           *
#  * this document, and assumes no liability of any kind, including the implied warranties of noninfringement,  *
#  * merchantability or fitness for particular purposes, with respect to the operation or use of AMD            *
#  * hardware, software or other products described herein.  No license, including implied or                   *
#  * arising by estoppel, to any intellectual property rights is granted by this document.  Terms and           *
#  * limitations applicable to the purchase or use of AMD’s products are as set forth in a signed agreement     *
#  * between the parties or in AMD's Standard Terms and Conditions of Sale. GD-18                               *
#  *                                                                                                            *
#  **************************************************************************************************************
#<copyright-disclaimer-end>
#<copyright-disclaimer-end>
#<copyright-disclaimer-end>
#<copyright-disclaimer-end>
