###########################################################################
## Report CDC Completer Script
###########################################################################

# load the standard helper file
source ../../fpgaSupport_scripts/script1.tcl
source ../../fpgaSupport_scripts/script2.tcl

# project constants
set verbose 	1
set tcName 		Report_CDC 
set demoOrLab 	completed
set projName 	wave_gen

 ## *********** Step 1 : Copying the project ***********
proc copyProject {} {

variable platform
variable language
variable tcName
variable demoOrLab 
variable projName 
variable TRAINING_PATH

# Set variables for source and destination paths
set sourcePath $TRAINING_PATH/$tcName/lab/$platform
set destPath $TRAINING_PATH/$tcName/completed/$platform

# Create the destination directories if they don't exist
file mkdir $destPath

# Copy the zip file to the destination directory
file copy -force $sourcePath/$language.zip $destPath
puts "sourcePath: $sourcePath"
puts "destPath: $destPath"

exec unzip $TRAINING_PATH/$tcName/completed/$platform/$language.zip -d $TRAINING_PATH/$tcName/completed/$platform/$language
}
 ## *********** Step 2 : Opening a project ***********

proc openProject {} {
variable platform
variable language
variable tcName
variable demoOrLab 
variable projName 
variable TRAINING_PATH

# Add the platform and language combination that you want 
set isLangNotSelected [string compare -nocase $language "undefined"]
set isPlatNotSelected [string compare -nocase $platform "undefined"]
   
if {$isLangNotSelected} {
      puts "Please type: use VHDL | Verilog"
      puts "   then rerun the projectCreate"
} 
if {$isPlatNotSelected} {
      puts "Please type: use KCU105 | KC705 | KC7xx"
      puts "   then rerun the projectCreate"
}

# Open a project
set projName.xpr {append $projName .xpr}
open_project $TRAINING_PATH/$tcName/$demoOrLab/$platform/$language/$projName.xpr
}

 ## *********** Step 3 : Analyzing the CDC Report ***********

proc analyzingReport {} {
variable platform
variable language
variable tcName
variable demoOrLab 
variable projName
variable TRAINING_PATH 

# Open Synthesized Design
open_run synth_1 -name synth_1

# Report CDC
report_cdc -name cdc_1

# View Schematic
show_schematic -name {Schematic - Clock Domain Crossing} -pin_pairs [list [get_pins rst_gen_i0/reset_bridge_clk_rx_i0/rst_meta_reg/Q] [get_pins rst_gen_i0/reset_bridge_clk_rx_i0/rst_dst_reg/D] [get_pins rst_gen_i0/reset_bridge_clk_rx_i0/rst_meta_reg/Q] [get_pins rst_gen_i0/reset_bridge_clk_rx_i0/rst_dst_reg/D] [get_ports rst_pin] [get_pins IBUF_rst_i0/INBUF_INST/PAD] [get_pins IBUF_rst_i0/INBUF_INST/O] [get_pins IBUF_rst_i0/IBUFCTRL_INST/I] [get_pins IBUF_rst_i0/IBUFCTRL_INST/O] [get_pins clk_gen_i0/rst_meta_i_1/I0] [get_pins clk_gen_i0/rst_meta_i_1/O] [get_pins rst_gen_i0/reset_bridge_clk_rx_i0/rst_meta_reg/PRE] [get_ports clk_pin_p] [get_pins clk_gen_i0/clk_core_i0/inst/clkin1_ibufds/DIFFINBUF_INST/DIFF_IN_P] [get_pins clk_gen_i0/clk_core_i0/inst/clkin1_ibufds/DIFFINBUF_INST/O] [get_pins clk_gen_i0/clk_core_i0/inst/clkin1_ibufds/IBUFCTRL_INST/I] [get_pins clk_gen_i0/clk_core_i0/inst/clkin1_ibufds/IBUFCTRL_INST/O] [get_pins clk_gen_i0/clk_core_i0/inst/mmcme3_adv_inst/CLKIN1] [get_pins clk_gen_i0/clk_core_i0/inst/mmcme3_adv_inst/CLKOUT0] [get_pins clk_gen_i0/clk_core_i0/inst/clkout1_buf/I] [get_pins clk_gen_i0/clk_core_i0/inst/clkout1_buf/O] [get_pins rst_gen_i0/reset_bridge_clk_rx_i0/rst_meta_reg/C] ]

# Copy the reset_bridge.v file from support directory
	set completedfilePath $TRAINING_PATH/$tcName/support
    set projName.srcs {append $projName .srcs}
	set path $TRAINING_PATH/$tcName/$demoOrLab/$platform/$language/$projName.srcs/sources_1/imports/
	file copy -force $completedfilePath/reset_bridge_Report_CDC.v $path/reset_bridge_Report_CDC.v

set synth 1 
reset_run synth_1

# Calling the proc which runs Synthesis
synthesisRun 
	
# Report CDC
report_cdc -name cdc_1

# Open Schematic
show_schematic -name {Schematic - Clock Domain Crossing} -pin_pairs [list [get_pins clkx_nsamp_i0/meta_harden_bus_new_i0/signal_meta_reg/Q] [get_pins clkx_nsamp_i0/meta_harden_bus_new_i0/signal_dst_reg/D] [get_pins clkx_nsamp_i0/meta_harden_bus_new_i0/signal_meta_reg/Q] [get_pins clkx_nsamp_i0/meta_harden_bus_new_i0/signal_dst_reg/D] [get_pins clkx_nsamp_i0/bus_new_stretch_src_reg/Q] [get_pins clkx_nsamp_i0/meta_harden_bus_new_i0/signal_meta_reg/D] ]

show_objects -name find_1 [get_nets -hierarchical -top_net_of_hierarchical_group "*signal_dst*" ]
show_objects -name find_2 [get_nets -hierarchical -top_net_of_hierarchical_group "*signal_meta*" ]

set_property ASYNC_REG true [get_cells -hierarchical -filter { NAME =~  "*signal_meta*" || NAME =~  "*signal_dst*" } ]
save_constraints

set impl 1 

# Calling the proc which runs implementation
implementationRun 

# Run CDC 
report_cdc -name cdc_1

# Find FDRE
show_objects -name find_2 [get_cells -hierarchical -filter { PRIMITIVE_TYPE == REGISTER.SDR.FDRE } ]

# Generate the Mean Time before Failure (MTBF) report 
report_synchronizer_mtbf

}

 ## ********** Running only the steps that are required with Make **************

proc make {stopAt} {

   puts "Running until the step $stopAt"
   set limit [string tolower $stopAt]
   switch $limit {
      step1  { copyProject }
	  step2  { make step1; openProject }
      step3  { make step2; analyzingReport }
      all    { make step3 }
      default { 
         puts "Call the make proc, Should be make step*" 
			  }	
	}	
}

#<copyright-disclaimer-start>
#<copyright-disclaimer-start>
#  **************************************************************************************************************
#  * © 2025 Advanced Micro Devices, Inc. All rights reserved.                                                   *
#  * DISCLAIMER                                                                                                 *
#  * The information contained herein is for informational purposes only, and is subject to change              *
#  * without notice. While every precaution has been taken in the preparation of this document, it              *
#  * may contain technical inaccuracies, omissions and typographical errors, and AMD is under no                *
#  * obligation to update or otherwise correct this information.  Advanced Micro Devices, Inc. makes            *
#  * no representations or warranties with respect to the accuracy or completeness of the contents of           *
#  * this document, and assumes no liability of any kind, including the implied warranties of noninfringement,  *
#  * merchantability or fitness for particular purposes, with respect to the operation or use of AMD            *
#  * hardware, software or other products described herein.  No license, including implied or                   *
#  * arising by estoppel, to any intellectual property rights is granted by this document.  Terms and           *
#  * limitations applicable to the purchase or use of AMD’s products are as set forth in a signed agreement     *
#  * between the parties or in AMD's Standard Terms and Conditions of Sale. GD-18                               *
#  *                                                                                                            *
#  **************************************************************************************************************
#<copyright-disclaimer-end>
#<copyright-disclaimer-end>
#<copyright-disclaimer-end>
#<copyright-disclaimer-end>
