###########################################################################
## Source Synchronous IO Timing Completer Script
###########################################################################

# load the standard helper file
source ../../fpgaSupport_scripts/script1.tcl
source ../../fpgaSupport_scripts/script2.tcl

# project constants
set verbose 	1
set tcName 	SourceSyncIO
set demoOrLab 	completed
set projName 	ddr_example_design

 ## *********** Step 1 : Copying the project ***********
proc copyProject {} {

variable platform
variable language
variable tcName
variable demoOrLab 
variable projName 
variable TRAINING_PATH

# Set variables for source and destination paths
set sourcePath $TRAINING_PATH/$tcName/lab/$platform
set destPath $TRAINING_PATH/$tcName/completed/$platform

# Create the destination directories if they don't exist
file mkdir $destPath

# Copy the zip file to the destination directory
file copy -force $sourcePath/$language.zip $destPath
puts "sourcePath: $sourcePath"
puts "destPath: $destPath"

exec unzip $TRAINING_PATH/$tcName/completed/$platform/$language.zip -d $TRAINING_PATH/$tcName/completed/$platform/$language
}

 ## *********** Step 1 : Opening a project, and adding the IO constraints ***********

proc openProject {} {
variable platform
variable language
variable tcName
variable demoOrLab 
variable projName 
variable TRAINING_PATH

# Add the platform and language combination that you want 
set isLangNotSelected [string compare -nocase $language "undefined"]
set isPlatNotSelected [string compare -nocase $platform "undefined"]
   
if {$isLangNotSelected} {
      puts "Please type: use VHDL | Verilog | netlist"
      puts "   then rerun the projectCreate"
}
if {$isPlatNotSelected} {
      puts "Please type: use KCU105 | KC705 | KC7xx"
      puts "   then rerun the projectCreate"
}

# Open a project
set projName.xpr {append $projName .xpr}
open_project $TRAINING_PATH/$tcName/$demoOrLab/$platform/$language/$projName.xpr


open_run impl_1 -name impl_1

set projName.srcs {append $projName .srcs}
set_property target_constrs_file $TRAINING_PATH/$tcName/$demoOrLab/$platform/$language/$projName.srcs/constrs_1/imports/SourceSyncIO/ddr_example_design.xdc [current_fileset -constrset]

# Adding the Constraints for either 7 series or Ultra-Scale board
set_input_delay -clock [get_clocks rxclk] -max 1.250 [get_ports {{rxdata[0]} {rxdata[1]}}]
set_input_delay -clock [get_clocks rxclk] -min 1.000 [get_ports {{rxdata[0]} {rxdata[1]}}]
set_input_delay -clock [get_clocks rxclk] -clock_fall -max -add_delay 1.250 [get_ports {{rxdata[0]} {rxdata[1]}}]
set_input_delay -clock [get_clocks rxclk] -clock_fall -min -add_delay 1.000 [get_ports {{rxdata[0]} {rxdata[1]}}]

save_constraints -force
close_design
}


 ## *********** Step 2 : Implementing the design and running report_datasheet  ***********

proc Impl_genReport {} {

set impl 1
reset_run impl_1

# Calling the proc which runs implementation
implementationRun

report_datasheet
}  


 ## *********** Step 3 : Adjusting the delay and verify  ***********

proc adjustDelay {} {
variable platform
variable language
variable tcName
variable demoOrLab 
variable projName 
variable TRAINING_PATH

set projName.srcs {append $projName .srcs}
set_property target_constrs_file $TRAINING_PATH/$tcName/$demoOrLab/$platform/$language/$projName.srcs/constrs_1/imports/SourceSyncIO/ddr_example_design.xdc [current_fileset -constrset]

# Adjusting the delay and verifying for either 7 series or Ultra-Scale board
set_property DELAY_FORMAT {COUNT} [get_cells IDELAY_RxData*]
save_constraints

set_property DELAY_VALUE {100} [get_cells IDELAY_RxData*]
report_datasheet

set_property DELAY_VALUE {270} [get_cells IDELAY_RxData*]
report_datasheet
 
save_constraints

reset_run impl_1

# Calling the proc which runs implementation
implementationRun

report_timing_summary -delay_type min_max -report_unconstrained -check_timing_verbose -max_paths 10 -input_pins -name timing_1
} 


 ## ********** Running only the steps that are required with Make **************

proc make {stopAt} {

   puts "Running until the step $stopAt"
   set limit [string tolower $stopAt]
   switch $limit {
      step1  { copyProject }
	  step2  { make step1; openProject }
      step3  { make step2; Impl_genReport }
      step4  { make step3; adjustDelay }
	  all    { make step4 }
      default { 
         puts "Call the make proc, Should be make step*" 
			  }	
	}	
}




#<copyright-disclaimer-start>
#<copyright-disclaimer-start>
#  **************************************************************************************************************
#  * © 2025 Advanced Micro Devices, Inc. All rights reserved.                                                   *
#  * DISCLAIMER                                                                                                 *
#  * The information contained herein is for informational purposes only, and is subject to change              *
#  * without notice. While every precaution has been taken in the preparation of this document, it              *
#  * may contain technical inaccuracies, omissions and typographical errors, and AMD is under no                *
#  * obligation to update or otherwise correct this information.  Advanced Micro Devices, Inc. makes            *
#  * no representations or warranties with respect to the accuracy or completeness of the contents of           *
#  * this document, and assumes no liability of any kind, including the implied warranties of noninfringement,  *
#  * merchantability or fitness for particular purposes, with respect to the operation or use of AMD            *
#  * hardware, software or other products described herein.  No license, including implied or                   *
#  * arising by estoppel, to any intellectual property rights is granted by this document.  Terms and           *
#  * limitations applicable to the purchase or use of AMD’s products are as set forth in a signed agreement     *
#  * between the parties or in AMD's Standard Terms and Conditions of Sale. GD-18                               *
#  *                                                                                                            *
#  **************************************************************************************************************
#<copyright-disclaimer-end>
#<copyright-disclaimer-end>
#<copyright-disclaimer-end>
#<copyright-disclaimer-end>
