###########################################################################
# Timing Simulation Completer Script
###########################################################################

# Load the standard helper files
source ../../fpgaSupport_scripts/script1.tcl
source ../../fpgaSupport_scripts/script2.tcl

# Project constants
set verbose 1
set tcName TimingSimulation
set demoOrLab completed
set projName MyCntr
set platform KCU105

## *********** Step 1 : Copying the project ***********
# This procedure copies the project files from the source to the destination directory
proc copyProject {} {
    variable platform
    variable language
    variable tcName
    variable demoOrLab 
    variable projName 
    variable TRAINING_PATH

    # Set variables for source and destination paths
    set sourcePath $TRAINING_PATH/$tcName/lab
    set destPath $TRAINING_PATH/$tcName/completed

    # Create the destination directories if they don't exist
    file mkdir $destPath

    # Copy the zip file to the destination directory
    file copy -force $sourcePath/$platform.zip $destPath
    puts "sourcePath: $sourcePath"
    puts "destPath: $destPath"

    # Unzip the project files in the destination directory
    exec unzip $TRAINING_PATH/$tcName/completed/$platform.zip -d $TRAINING_PATH/$tcName/completed/$platform
}

## *********** Step 2 : Opening a project, opening an implemented design ***********
# This procedure opens the specified project and its implemented design
proc openProject {} {
    variable tcName
    variable demoOrLab 
    variable projName 
    variable TRAINING_PATH

    # Open the project file
    set projName.xpr {append $projName .xpr}
    open_project $TRAINING_PATH/$tcName/$demoOrLab/KCU105/$projName.xpr

    # Open the implemented design
    open_run impl_1

    # Explore the simulation settings at this step (comment indicating manual inspection point)
}

## *********** Step 3 : Running timing simulation and verifying the delays ***********
# This procedure runs a post-implementation timing simulation and verifies the delays
proc runSim {} {
    # Running a post-implementation timing simulation
    launch_simulation -mode post-implementation -type timing

    # Add the markers manually and verify the delays as per the SDF file (comment indicating manual inspection point)
}

## ********** Running only the steps that are required with Make **************
# This procedure controls the flow of the script, allowing selective execution of steps
proc make {stopAt} {
    puts "Running until the step $stopAt"
    set limit [string tolower $stopAt]
    switch $limit {
        step1  { copyProject }
        step2  { make step1; openProject }
        step3  { make step2; runSim }
        all    { make step3 }
        default { 
            puts "Call the make proc, should be make step*" 
        }
    }    
}

#<copyright-disclaimer-start>
#<copyright-disclaimer-start>
#  **************************************************************************************************************
#  * © 2025 Advanced Micro Devices, Inc. All rights reserved.                                                   *
#  * DISCLAIMER                                                                                                 *
#  * The information contained herein is for informational purposes only, and is subject to change              *
#  * without notice. While every precaution has been taken in the preparation of this document, it              *
#  * may contain technical inaccuracies, omissions and typographical errors, and AMD is under no                *
#  * obligation to update or otherwise correct this information.  Advanced Micro Devices, Inc. makes            *
#  * no representations or warranties with respect to the accuracy or completeness of the contents of           *
#  * this document, and assumes no liability of any kind, including the implied warranties of noninfringement,  *
#  * merchantability or fitness for particular purposes, with respect to the operation or use of AMD            *
#  * hardware, software or other products described herein.  No license, including implied or                   *
#  * arising by estoppel, to any intellectual property rights is granted by this document.  Terms and           *
#  * limitations applicable to the purchase or use of AMD’s products are as set forth in a signed agreement     *
#  * between the parties or in AMD's Standard Terms and Conditions of Sale. GD-18                               *
#  *                                                                                                            *
#  **************************************************************************************************************
#<copyright-disclaimer-end>
#<copyright-disclaimer-end>
#<copyright-disclaimer-end>
#<copyright-disclaimer-end>
