
# Global timing constraints
create_clock -period 8.000 -name clk_pin_p [get_ports clk_pin_p]

set_input_delay -clock clk_pin_p 1.200 [get_ports rst_pin]
set_input_delay -clock clk_pin_p -min 1.000 [get_ports rst_pin]
set_input_delay -clock clk_pin_p 0.200 [get_ports rxd_pin]
set_input_delay -clock clk_pin_p -min -0.200 [get_ports rxd_pin]

create_clock -period 8.258 -name clk_tx_virtual
set_input_delay -clock clk_tx_virtual 1.000 [get_ports lb_sel_pin]
set_input_delay -clock clk_tx_virtual -min 1.000 [get_ports lb_sel_pin]
set_output_delay -clock clk_tx_virtual 0.200 [get_ports {txd_pin spi_clk_pin {led_pins[*]}}]

# Path specific timing constraints
set_output_delay -clock clk_tx_virtual -clock_fall -add_delay 1.200 [get_ports spi_clk_pin]
set_false_path -from [get_clocks clk_out1_clk_core] -to [get_clocks clk_out2_clk_core]

create_generated_clock -name spi_clk -source [get_pins dac_spi_i0/out_ddr_flop_spi_clk_i0/ODDRE1_inst/C] -divide_by 1 [get_ports spi_clk_pin]
set_output_delay -clock [get_clocks spi_clk] -max 1.200 [get_ports {spi_mosi_pin dac_cs_n_pin dac_clr_n_pin}]
set_output_delay -clock [get_clocks spi_clk] -min 0.800 [get_ports {spi_mosi_pin dac_cs_n_pin dac_clr_n_pin}]

set_multicycle_path -from [get_cells {cmd_parse_i0/send_resp_data_reg[*]}] -to [get_cells {resp_gen_i0/to_bcd_i0/bcd_out_reg[*]}] 2
set_multicycle_path -hold -from [get_cells {cmd_parse_i0/send_resp_data_reg[*]}] -to [get_cells {resp_gen_i0/to_bcd_i0/bcd_out_reg[*]}] 1

set_multicycle_path -from [get_cells uart_rx_i0/uart_rx_ctl_i0/* -filter {IS_SEQUENTIAL == 1}] -to [get_cells uart_rx_i0/uart_rx_ctl_i0/* -filter {IS_SEQUENTIAL == 1}] 108
set_multicycle_path -hold -from [get_cells uart_rx_i0/uart_rx_ctl_i0/* -filter {IS_SEQUENTIAL == 1}] -to [get_cells uart_rx_i0/uart_rx_ctl_i0/* -filter {IS_SEQUENTIAL == 1}] 107

set_multicycle_path -from [get_cells uart_tx_i0/uart_tx_ctl_i0/* -filter {IS_SEQUENTIAL == 1}] -to [get_cells uart_tx_i0/uart_tx_ctl_i0/* -filter {IS_SEQUENTIAL == 1}] 108
set_multicycle_path -hold -from [get_cells uart_tx_i0/uart_tx_ctl_i0/* -filter {IS_SEQUENTIAL == 1}] -to [get_cells uart_tx_i0/uart_tx_ctl_i0/* -filter {IS_SEQUENTIAL == 1}] 107

create_generated_clock -name clk_samp -source [get_pins clk_gen_i0/BUFHCE_clk_samp_i0/I] -divide_by 32 [get_pins clk_gen_i0/BUFHCE_clk_samp_i0/O]

set_max_delay -from clkx_nsamp_i0/meta_harden_bus_new_i0/signal_meta_reg -to clkx_nsamp_i0/meta_harden_bus_new_i0/signal_dst_reg 2.000
set_max_delay -from clkx_pre_i0/meta_harden_bus_new_i0/signal_meta_reg -to clkx_pre_i0/meta_harden_bus_new_i0/signal_dst_reg 2.000
set_max_delay -from clkx_spd_i0/meta_harden_bus_new_i0/signal_meta_reg -to clkx_spd_i0/meta_harden_bus_new_i0/signal_dst_reg 2.000
set_max_delay -from lb_ctl_i0/debouncer_i0/meta_harden_signal_in_i0/signal_meta_reg -to lb_ctl_i0/debouncer_i0/meta_harden_signal_in_i0/signal_dst_reg 2.000
set_max_delay -from samp_gen_i0/meta_harden_samp_gen_go_i0/signal_meta_reg -to samp_gen_i0/meta_harden_samp_gen_go_i0/signal_dst_reg 2.000
set_max_delay -from uart_rx_i0/meta_harden_rxd_i0/signal_meta_reg -to uart_rx_i0/meta_harden_rxd_i0/signal_dst_reg 2.000

set_max_delay -from rst_gen_i0/reset_bridge_clk_rx_i0/rst_meta_reg -to rst_gen_i0/reset_bridge_clk_rx_i0/rst_dst_reg 2.000
set_max_delay -from rst_gen_i0/reset_bridge_clk_tx_i0/rst_meta_reg -to rst_gen_i0/reset_bridge_clk_tx_i0/rst_dst_reg 2.000
set_max_delay -from rst_gen_i0/reset_bridge_clk_samp_i0/rst_meta_reg -to rst_gen_i0/reset_bridge_clk_samp_i0/rst_dst_reg 2.000

set_false_path -from [get_ports rst_pin]
set_false_path -from [get_ports spi_clk_pin]
set_output_delay -clock [get_clocks clk_tx_virtual] 0.000 [get_ports {{led_pins[0]} {led_pins[1]} {led_pins[2]} {led_pins[3]} txd_pin}]


#set_clock_groups -name async_debug -asynchronous -group [get_clocks clk_out1_clk_core] -group [get_clocks clk_out2_clk_core]
#set_clock_groups -name async_debug -asynchronous -group [get_clocks clk_rx] -group [get_clocks clk_tx]

set_false_path -from [get_clocks clk_tx_virtual] -to [get_clocks -of_objects [get_pins clk_gen_i0/clk_core_i0/inst/mmcme4_adv_inst/CLKOUT1]]
set_false_path -from [get_clocks -of_objects [get_pins clk_gen_i0/clk_core_i0/inst/mmcme4_adv_inst/CLKOUT1]] -to [get_clocks clk_tx_virtual]
set_property C_CLK_INPUT_FREQ_HZ 125000000 [get_debug_cores dbg_hub]
set_property C_ENABLE_CLK_DIVIDER false [get_debug_cores dbg_hub]
set_property C_USER_SCAN_CHAIN 1 [get_debug_cores dbg_hub]
connect_debug_port dbg_hub/clk [get_nets clk_rx]



#<copyright-disclaimer-start>
#<copyright-disclaimer-start>
#  **************************************************************************************************************
#  * © 2025 Advanced Micro Devices, Inc. All rights reserved.                                                   *
#  * DISCLAIMER                                                                                                 *
#  * The information contained herein is for informational purposes only, and is subject to change              *
#  * without notice. While every precaution has been taken in the preparation of this document, it              *
#  * may contain technical inaccuracies, omissions and typographical errors, and AMD is under no                *
#  * obligation to update or otherwise correct this information.  Advanced Micro Devices, Inc. makes            *
#  * no representations or warranties with respect to the accuracy or completeness of the contents of           *
#  * this document, and assumes no liability of any kind, including the implied warranties of noninfringement,  *
#  * merchantability or fitness for particular purposes, with respect to the operation or use of AMD            *
#  * hardware, software or other products described herein.  No license, including implied or                   *
#  * arising by estoppel, to any intellectual property rights is granted by this document.  Terms and           *
#  * limitations applicable to the purchase or use of AMD’s products are as set forth in a signed agreement     *
#  * between the parties or in AMD's Standard Terms and Conditions of Sale. GD-18                               *
#  *                                                                                                            *
#  **************************************************************************************************************
#<copyright-disclaimer-end>
#<copyright-disclaimer-end>
#<copyright-disclaimer-end>
#<copyright-disclaimer-end>
